function [A,f0] = plinearize(f,x,x0)
% function [A,f0] = plinearize(f,x,x0)
%
% DESCRIPTION 
%   This function linearizes a polynomial f(x) about the point x0.
%   The linear Taylor series is f(x) ~ f(x0) + A*(x-x0)
%   where A_ij = (df_i/dx_j)(x0).
%
% INPUTS 
%   f: Vector field of polynomial system  (Ns-by-1 polynomial)
%   x: State  (Ns-by-1 vector of pvars)
%   x0: (Default: x0=0)
%     
% OUTPUTS
%   A: Linearization of the polynomial around x=x0.
%   f0: f evaluated at x0
%
% SYNTAX
%   [A,f0] = plinearize(f,x)
%   [A,f0] = plinearize(f,x,x0)
%
% EXAMPLE
%   pvar x1 x2;
%   x = [x1;x2];
%   f = [-2*x1+x2+x1^2; x1-3*x2]
%   x0 = [3;4];
%   A = plinearize(f,x,x0)
%

% PJS 9/29/2009   Initial Coding

% Error Checking
if nargin==2
    Nx = length(x);
    x0 = zeros(Nx,1);
end

% Evaluate function at trim
f0 = subs(f,x,x0);

% Shift variable so trim is at the origin
fshifted = subs(f,x,x0+x);

% Linearize shifted function about origin
tol = [];
deg = 1;
flin = cleanpoly(fshifted,tol,deg);

% Extract  matrix: f(x0+x) ~ f(x0) + A*x  with A a double matrix
A = double(jacobian(flin,x));
